static cFileIn,nLineLen,nCounter,nLinesToGo

#include "SLVRCLIP.CH"
#include "BOX.CH"

#define SOH 1
#define EOT 4
#define ACK 6 
#define NAK 21
#define ESCAPE 27
#define CRLF chr(13)+chr(10)

#define TERMINATOR_REACHED   1
#define MAX_LENGTH_REACHED   2
#define GOT_EOT              3
#define ESCAPE_HIT         -27
#define TIMED_OUT           -1

function pit_recv(nPort)

local lSuccess := .f.

SETCOLOR("G+/B")

dispbox(6,10,16,69, B_DOUBLE_SINGLE +CHR(32))

setcolor("gr+/r")

@ 6,16 say ' HITCOM ** Handheld Intelligent Terminal Communications '
@ 16,26 say ' Public Domain by Phil Barnett '

setcolor("gr+/b")

@ 8,20 say 'Comm Port: COM'+str(nPort+1,1)
@ 8,45 say 'Port Speed: ' + alltrim(str(swgetbaud(nPort)))

stat('Handshake')
if r_handshak(nPort)
  stat('Packet Sizing')
  if r_sizepack(nPort)
    stat('Normal Receive')
    if r_transmit(nPort)
      stat('Success')
      lSuccess := .t.
    else
      Stat('Failed in Data Transfer')
    endif
  else
    stat('Failed in Packet Sizing')
  endif
else
  Stat('Failed in Hand Shake')
endif

return lSuccess

*********************************

static function r_handshak(nPort)

local nTries := 0
local cBuffer
local nRetval

swflushtx(nPort)
swflushrx(nPort)
swwait(1*SWSECOND)

do while nTries++ < 10
   swtxchar(nPort,NAK)
   
   cBuffer := ''
   nRetval := buff2string(nPort,@cBuffer,100,SOH,10)
   
   if nRetval == TERMINATOR_REACHED .or. nRetval == ESCAPE_HIT
     exit
   endif
enddo

return nRetval == TERMINATOR_REACHED

*************************************

static function r_sizepack(nPort)

local lSuccess := .f.
local nTries   := 0
local cBuffer,nRetval,crc,data,sentcrc

do while nTries++ < 10
   cBuffer := ''
   nRetval := buff2string(nPort,@cBuffer,132,-1,10)

   if nRetval == MAX_LENGTH_REACHED
     if left(cBuffer,2) == chr(0)+chr(255)
       sentcrc  := right(cBuffer,2)
       data     := substr(cBuffer,3,128)
       sentcrc  := swmakeword(asc(right(sentcrc,1)),asc(left(sentcrc,1)))
       crc      := crc16(data)
       if sentcrc == crc
         nLineLen   := BIN2I(left(data,2))
         nLinesToGo := BIN2L(substr(data,3,4))
         cFileIn    := substr(data,7,12) // Just 12 characters in DOS. No Path
         nCounter   := 1   // this is the expected first package number

         @ 10,20 say 'File: '+cFileIn
         @ 10,45 say 'Record Length: '+alltrim(str(nLineLen))
         rtg(nLinesToGo)

         lSuccess   := .t.
         swtxchar(nPort,ACK)
         exit
       else
         stat('Bad CRC')
         swtxchar(nPort,NAK)
       endif
     else
       Stat('Packet count was wrong')
       swtxchar(nPort,NAK)
     endif
   else
     swtxchar(nPort,NAK)
     stat('Packet wrong length')
   endif
enddo

Return lSuccess

*************************************

static function r_transmit(nPort)

local lSuccess := .f.
local nTries   := 0
local cBuffer,nRetval,crc,data,sentcrc,lastcounter

local handle := fcreate(cFileIn,0)  // this will automatically overwrite !!

init_progress(21,nLinesToGo)

do while nTries++ < 10
   cBuffer := ''
   nRetval := buff2string(nPort,@cBuffer,nLineLen+5,-1,10)
   
   if nRetval == GOT_EOT
     swtxchar(nPort,ACK)  // ACK it
     lSuccess := .t.
     exit
   endif
   
   if nRetval == MAX_LENGTH_REACHED
     
     if left(cBuffer,1) == chr(SOH)
     
       // Make sure it is the next packet...

       lastcounter := nCounter - 1
       if lastcounter < 0
         lastcounter := 255
       endif
       if substr(cBuffer,2,2) == chr(lastCounter)+chr(255-lastCounter)
         stat('Got same packet twice')
         swtxchar(nPort,ACK)  // ACK it again...
         loop
       endif

       // OK, we're on the current packet
                                                                            
       if substr(cBuffer,2,2) == chr(nCounter)+chr(255-nCounter)
         sentcrc  := right(cBuffer,2)
         data     := substr(cBuffer,4,nLineLen)
         sentcrc  := swmakeword(asc(right(sentcrc,1)),asc(left(sentcrc,1)))
         crc      := crc16(data)
         if sentcrc == crc
           // Write it...
           fwrite(handle,data+CRLF)
           stat("Normal Receive")
           swtxchar(nPort,ACK)
           progress()
           rtg(--nLinesToGo)
           nCounter ++
           if nCounter > 255
             nCounter := 0
           endif
           nTries := 0
         else
           stat('Bad CRC')
           swtxchar(nPort,NAK)
         endif
       else
         stat('Packet Count was wrong')
         swtxchar(nPort,NAK)
       endif
     else
         STAT("Packet did not start with SOH")
         swtxchar(nPort,NAK)
     endif
   else
     stat('Packet wrong length')
     swtxchar(nPort,NAK)
   endif
enddo

kill_progress()

fclose(handle)

Return lSuccess

*************************************

static function r_terminat(nPort)

swtxchar(nPort,ACK)  // ACK the terminate function

return .t.

*************************************

function buff2string(nPort,cBuffer,nBuffLen,nTerminator,nTimeOut)

local reason := TIMED_OUT
local inchar
local charcount := 0

cBuffer := ''

SWCrtTimer(1,nTimeOut*18.2)

do while SWGTimer(1) > 0

  if !swrxempty(nPort)
  
    inchar := swrxchar(nPort)
    cBuffer += chr(inchar)
    charcount ++
    
    if charcount == 1
      if inchar == EOT
        reason := GOT_EOT
        exit
      endif
    endif
    
    if inchar == nTerminator
      reason := TERMINATOR_REACHED
      exit
    endif
    
    if len(cBuffer) >= nBufflen
      reason := MAX_LENGTH_REACHED
      exit
    endif

  endif
  
  if inkey() == ESCAPE .or. lastkey() == ESCAPE
    reason := ESCAPE_HIT
    exit
  endif
  
enddo

SWDSyTimer(1)

return reason

