static cFileOut,nLineLen,nCounter,RecsToGo

#include "SLVRCLIP.CH"
#include "BOX.CH"

#define SOH 1
#define EOT 4
#define ACK 6 
#define NAK 21
#define ESCAPE 27
#define CRLF chr(13)+chr(10)

#define TERMINATOR_REACHED 1
#define MAX_LENGTH_REACHED 2
#define ESCAPE_HIT         -27
#define TIMED_OUT          -1

function pit_tran(nPort,cFileName,nRecSize,nNumRecs)

local lSuccess := .f.

SETCOLOR("G+/B")

dispbox(6,10,16,69, B_DOUBLE_SINGLE +CHR(32))

setcolor("gr+/r")

@ 6,12 say ' HITCOM ** Handheld Intelligent Terminal Communications '
@ 16,25 say ' Public Domain by Phil Barnett '

setcolor("gr+/b")

@ 8,20 say 'Comm Port: COM'+str(nPort+1,1)
@ 8,45 say 'Port Speed: ' + alltrim(str(swgetbaud(nPort)))
@ 10,20 say 'File: '+cFileName
@ 10,45 say 'Record Length: '+alltrim(str(nRecSize))
rtg(nNumRecs)

STAT('Hand Shake')
if t_handshak(nPort)
  STAT('Packet Sizing')
  if t_sizepack(nPort,cFileName,nRecSize,nNumRecs)
    STAT('Normal Transmission')
    if t_transmit(nPort)
      STAT('Termination')
      if t_terminate(nPort)
        STAT('Success')
        lSuccess := .t.
      else
        STAT('Failed in Termination')
      endif
    else
      STAT('Failed in Data Transfer')
    endif
  else
    STAT('Failed in Packet Sizing')
  endif
else
  STAT('Failed in Hand Shake')
endif

return lSuccess

function t_handshak(nPort)

local nTries := 0
local cBuffer,nRetval

nCounter := 1
swflushtx(nPort)
swflushrx(nPort)
swwait(1*SWSECOND)

do while nTries++ < 10 .and. lastkey() != ESCAPE
   cBuffer := space(100)
   nRetval := buff2string(nPort,@cBuffer,100,NAK,10)
   if nRetval == TERMINATOR_REACHED
     exit
   endif
enddo

return nRetval == TERMINATOR_REACHED

*************************************

function t_sizepack(nPort,cFileName,nRecSize,nNumRecs)

local nTries := 0
local data,packet,timeout,nThisChar,crc

cFileOut := cFileName  // assign static for later use
nLineLen := nRecSize   // assign static for later use
RecsToGo := nNumRecs   // assign static for later use

data := I2BIN(nRecSize)+L2BIN(nNumRecs)+left(trim(cFileName)+replicate(chr(0),32),32)+replicate(chr(0),90)

crc := crc16(data)

packet := chr(SOH)+chr(0)+chr(255)+data+chr(swlowbyte(crc))+chr(swhighbyte(crc))

return send4ack(nPort,packet)

*************************************

function t_transmit(nPort)

local nBuffLen := 1024
local cReadString := ''
local packet,nHandle,nBytesRead,cBuffer,crlfat,data,crc

nHandle := fopen(cFileOut,0)

init_progress(20,RecsToGo)

nBytesRead := nBuffLen
do while nBytesRead == nBuffLen
  cBuffer := space(nBuffLen)
  nBytesRead := fread(nHandle,@cBuffer,nBuffLen)
  cReadString += left(cBuffer,nBytesRead)
  crlfat := at(CRLF,cReadString)
  do while crlfat > 0
    data := pad(left(cReadString,nLineLen),nLineLen)
    
    cReadString := substr(cReadString,crlfat+2)
    
    crc := crc16(data)
    
    packet := chr(SOH) + chr(nCounter) + chr(255-nCounter) + data + chr(swlowbyte(crc))+chr(swhighbyte(crc))
    
    /* Transmit data packet, and wait for ack or nak */
    
    if send4ack(nPort,packet)
      RecsToGo --
      rtg(RecsToGo)
      progress()
      nCounter ++
      if nCounter > 255
        nCounter := 0
      endif
    else
      kill_progress()
      return .f.
    endif
    
    crlfat := at(CRLF,cReadString)
  enddo

enddo

kill_progress()

return .t.  /* Made it through the entire file */

*************************************

function t_terminat(nPort)

return send4ack(nPort,replicate(chr(EOT),4))

*************************************

function send4ack(nPort,data)

local nTries := 0
local timeout,nThisChar

do while nTries++ < 10 .and. lastkey() != ESCAPE

  if nTries == 1
    STAT('Normal Transmission')
  else
    stat(" Retry # "+str(nTries,2))
  endif
   
  send2Buffer(nPort,data)
  
  SWCrtTimer(1,182)
  do while SWGTimer(1) > 0
    if swgrxcount(nPort) > 0
      nThisChar := swrxchar(nPort)
    else
      nThisChar := 0
    endif
    if nThisChar == ACK .or. nThisChar == NAK .or. inkey() == ESCAPE
      exit
    endif
  enddo
  SWDSyTimer(1)
  
  if nThisChar == ACK
    exit
  endif
enddo

return nThisChar == ACK

*************************************

function send2buffer(nPort,data)

local datalen := len(data)
local x

do while swgtxspace(nPort) < datalen  // make sure there is room in buffer
enddo

for x := 1 to datalen
  swtxchar(nPort,asc(substr(data,x,1)))
next

return NIL

*************************************

function stat(string)

@ 12,20 say 'Status: '+pad(string,30)

return NIL

*************************************

function rtg(recsleft)

@ 14,20 say 'Records to go: '+str(recsleft,6)

return NIL

